@@ -43,17 +43,16 @@ def comprehension_option(block_dets, *, repeat=False):
         return None
 
     title = layout(f"""\
-
-        ### Possible option of using a {comp_type}
-
-        """)
+    ### Possible option of using a {comp_type}
+    """)
     if not repeat:
         option = layout(f"""\
-            Simple for loops can sometimes be replaced with comprehensions. In
-            this case a simple reading of the code suggests a {comp_type} might
-            be possible. Of course, only use a comprehension if it makes your
-            code easier to understand.
-            """)
+
+        Simple for loops can sometimes be replaced with comprehensions. In this
+        case a simple reading of the code suggests a {comp_type} might be
+        possible. Of course, only use a comprehension if it makes your code
+        easier to understand.
+        """)
     else:
         option = ''
 
@@ -145,45 +144,35 @@ def for_index_iteration(block_dets, *, repeat=False):
         return None
 
     summary = layout(f"""\
+    ### Possible option of using direct iteration
 
-        ### Possible option of using direct iteration
-
-        It looks like your snippet iterates through `{iterable_name}` using
-        indexes. In Python you can iterate directly which is much easier.
-        """)
+    It looks like your snippet iterates through `{iterable_name}` using indexes.
+    In Python you can iterate directly which is much easier.
+    """)
     if not repeat:
         examples = (
-            layout(f"""\
-
-                For example, instead of:
-
-                """)
+            layout("""\
+            For example, instead of:
+            """)
             +
             layout(f"""\
-
-                for {index_name} in range(len({iterable_name})):
-                    print({iterable_name}[{index_name}])
-
-                """, is_code=True)
+            for {index_name} in range(len({iterable_name})):
+                print({iterable_name}[{index_name}])
+            """, is_code=True)
+            +
+            layout("""\
+            you can directly iterate as follows:
+            """)
             +
             layout(f"""\
+            for item in {iterable_name}:  ## item should be replaced with a more useful name
+                print(item)
 
-                you can directly iterate as follows:
-
-                """)
+            """, is_code=True)
             +
-            layout(f"""\
-
-                for item in {iterable_name}:  ## item should be replaced with a more useful name
-                    print(item)
-
-                """, is_code=True)
-            +
-            layout(f"""\
-
-                which is considered more pythonic i.e. good.
-
-                """)
+            layout("""\
+            which is considered more pythonic i.e. good.
+            """)
         )
     else:
         examples = ''
@@ -193,6 +182,84 @@ def for_index_iteration(block_dets, *, repeat=False):
     }
     return message
 
+@filt_block_advisor(xpath=FOR_XPATH, warning=True)
+def for_else(block_dets, *, repeat=False):
+    """
+    Look for the for-else construct and warn about its safe usage.
+    """
+    for_els = block_dets.element.xpath(FOR_XPATH)
+    has_for_else = False
+    for for_el in for_els:
+        for_else_els = for_el.xpath('orelse')
+        non_empty_for_else_els = [el for el in for_else_els if el.getchildren()]
+        if non_empty_for_else_els:
+            has_for_else = True
+            break
+    if not has_for_else:
+        return None
+
+    title = layout("""\
+    ### Ambiguous `for-else` language feature used
+    """)
+    if not repeat:
+        problem = (
+            layout("""\
+
+            `for`-`else` is a useful language feature but it is so widely
+            misunderstood it is dangerous to use without special comment. For
+            example, in the snippet below do we pass through `else` because we
+            finished the looping or because we didn't?
+            """)
+            +
+            layout("""\
+            for i in range(size):
+                if i > 100:
+                    break
+            else:
+                pass
+            """, is_code=True)
+        )
+        solution = (
+            layout("""\
+
+            The correct answer, which as many as a third of people will guess
+            correctly ;-), is that we only follow the `else` path if we finished
+            the for-looping without breaking. The thinking is that we loop
+            through something to find something and exit the loop. `else`
+            applies when that search fails. But it is very easy to get the logic
+            confused. And given that programs are made to be read by humans we
+            obviously need to do something. We can either avoid the feature
+            altogether or we can add a brief comment each time (my preference).
+            For example:
+            """)
+            +
+            layout("""\
+            for i in range(size):
+                if i > 100:
+                    break
+            else:  # didn't break for-loop
+                pass
+            """, is_code=True)
+        )
+        extra_msg = layout("""\
+        #### Interesting discussion on `for`-`else`
+
+        <https://nedbatchelder.com/blog/201110/forelse.html>
+        One comment noted that the construct violates the principle of Least
+        Astonishment.
+        """)
+    else:
+        problem = ''
+        solution = ''
+        extra_msg = ''
+
+    message = {
+        conf.BRIEF: title + problem,
+        conf.MAIN: title + problem + solution,
+        conf.EXTRA: extra_msg,
+    }
+    return message
+
 @filt_block_advisor(xpath=FOR_XPATH)
 def nested_fors(block_dets, *, repeat=False):
     """
@@ -210,46 +277,41 @@ def nested_fors(block_dets, *, repeat=False):
         return None
 
     summary = layout("""\
+    ### Possible option of simplifying nested iteration
 
-        ### Possible option of simplifying nested iteration
-
-        Consider replacing nested iteration with `itertools.product`.
-
-        """)
+    Consider replacing nested iteration with `itertools.product`.
+    """)
     if not repeat:
         demo = (
             layout("""\
-                For example, you could replace:
-
-                """)
+            For example, you could replace:
+            """)
             +
             layout("""\
-                for person in persons:
-                    for pet in pets:
-                        for year in years:
-                            print(f"{person} might like a {pet} in {year}")
-                """, is_code=True)
+            for person in persons:
+                for pet in pets:
+                    for year in years:
+                        print(f"{person} might like a {pet} in {year}")
+            """, is_code=True)
             +
             layout("""\
-
-                with a version using `itertools.product`:
-
-                """)
+            with a version using `itertools.product`:
+            """)
             +
             layout("""\
-                from itertools import product
-                for person, pet, year in product(persons, pets, years):
-                    print(f"{person} might like a {pet} in {year}")
-                """, is_code=True)
+            from itertools import product
+            for person, pet, year in product(persons, pets, years):
+                print(f"{person} might like a {pet} in {year}")
+            """, is_code=True)
         )
         pros = layout("""\
 
-            Whether this is a good idea or not depends on your specific code but
-            using `product` has the advantage of reducing indentation. It also
-            semantically expresses the intention of the code - namely to look at
-            every option in the cartesian product of the different collections
-            of items.
-            """)
+        Whether this is a good idea or not depends on your specific code but
+        using `product` has the advantage of reducing indentation. It also
+        semantically expresses the intention of the code - namely to look at
+        every option in the cartesian product of the different collections of
+        items.
+        """)
     else:
         demo = ''
         pros = ''
