@@ -2,64 +2,24 @@
 Covers functions and methods.
 """
 from ..advisors import filt_block_advisor
-from ..ast_funcs import get_el_lines_dets
+from ..ast_funcs import get_danger_status, get_docstring_from_value, \
+    get_el_lines_dets
 from .. import conf, utils
-from ..utils import get_python_version, layout_comment as layout
-
-def get_danger_status_3_7(child_el):
-    if (child_el.tag == 'NameConstant'
-            and child_el.get('value') in ['True', 'False']):
-        danger_status = 'Boolean'
-    elif child_el.tag == 'Num' and child_el.get('n'):
-        danger_status = 'Number'
-    else:
-        danger_status = None
-    return danger_status
-
-def get_danger_status_3_8(child_el):
-    if child_el.tag == 'Constant':
-        val = child_el.get('value')
-        if val in ['True', 'False']:
-            danger_status = 'Boolean'
-        else:
-            try:
-                float(val)
-            except TypeError:
-                danger_status = None
-            else:
-                danger_status = 'Number'
-    else:
-        danger_status = None
-    return danger_status
-
-def get_docstring_from_value_3_7(first_value_el):
-    if first_value_el.tag != 'Str':
-        docstring = None
-    else:
-        docstring = first_value_el.get('s')
-    return docstring
-
-def get_docstring_from_value_3_8(first_value_el):
-    if first_value_el.tag != 'Constant':
-        docstring = None
-    elif first_value_el.get('type') != 'str':
-        docstring = None
-    else:
-        docstring = first_value_el.get('value')
-    return docstring
-
-python_version = get_python_version()
-if python_version in (conf.PY3_6, conf.PY3_7):
-    get_danger_status = get_danger_status_3_7
-    get_docstring_from_value = get_docstring_from_value_3_7
-elif python_version == conf.PY3_8:
-    get_danger_status = get_danger_status_3_8
-    get_docstring_from_value = get_docstring_from_value_3_8
-else:
-    raise Exception(f"Unexpected Python version {python_version}")
+from ..utils import get_nice_pairs, layout_comment as layout
 
 FUNC_DEFN_XPATH = 'descendant-or-self::FunctionDef'
 
+def get_mutable_status(default_el):
+    """
+    KISS - Only look for List, Dict, and Set. Python 3.6 as well as 3.8
+    compatible.
+    """
+    if default_el.tag in ['List', 'Dict', 'Set']:
+        mutable_status = default_el.tag
+    else:
+        mutable_status = None
+    return mutable_status
+
 def get_is_method(func_el):
     class_els = func_el.xpath('ancestor::ClassDef')
     if not class_els:
@@ -86,19 +46,28 @@ def get_overall_func_type_lbl(func_els):
         conf.FUNCTION_LBL if includes_plain_function else conf.METHOD_LBL)
     return overall_func_type_lbl
 
+def count_args(func_el):
+    """
+    Have to handle positional-only, keyword-only, and standard arguments. Not
+    counting any unpacked args (vararg and kwarg).
+    """
+    posonlyargs = func_el.xpath('args/arguments/posonlyargs/arg')  ## python 3.8+
+    args = func_el.xpath('args/arguments/args/arg')
+    kwonlyargs = func_el.xpath('args/arguments/kwonlyargs/arg')
+    all_args_n = len(posonlyargs + args + kwonlyargs)
+    return all_args_n
+
 def _get_arg_comment(func_el, *, repeat=False):
     """
-    Must cope with positional arguments, keyword arguments, keyword-only
-    arguments, packed positional arguments, and packed keyword arguments.
-    Trivial really ;-).
+    Must cope with positional-only arguments, positional arguments, keyword
+    arguments, keyword-only arguments, packed positional arguments, and packed
+    keyword arguments. Trivial really ;-).
 
     Comment should end without a full stop because calling code adds that to
     make the sentence structure more explicit.
     """
-    args = func_el.xpath('args/arguments/args/arg')
     vararg = func_el.xpath('args/arguments/vararg/arg')
     kwarg = func_el.xpath('args/arguments/kwarg/arg')
-    kwonlyargs = func_el.xpath('args/arguments/kwonlyargs/arg')
     has_packing = (vararg or kwarg)
     if has_packing:
         arg_comment = 'receives a variable number of arguments'
@@ -120,7 +89,7 @@ def _get_arg_comment(func_el, *, repeat=False):
                     "better name in a particular case the Python convention is "
                     "to call that dictionary 'kwargs'")
     else:
-        all_args_n = len(args + kwonlyargs)
+        all_args_n = count_args(func_el)
         if all_args_n:
             nice_n_args = utils.int2nice(all_args_n)
             arg_comment = (f"receives {nice_n_args} argument")
@@ -165,13 +134,13 @@ def _get_return_comment(func_type_lbl, return_elements, *,
         if not repeat:
             if keyword_returns_n > 1:
                 returns_comment += (
-                    f"s. Some people prefer {func_type_lbl}s to have one, and "
-                    "only one exit point for clarity. Others disagree e.g. "
-                    f"using early returns to short-circuit {func_type_lbl}s if "
-                    "initial validation of some sort makes it possible to avoid"
-                    f" the bulk of the {func_type_lbl}. Whatever approach you "
-                    f"take make sure your {func_type_lbl} is easy to reason "
-                    "about in terms of what it returns and where it exits")
+                f"s. Some people prefer {func_type_lbl}s to have one, and only "
+                "one exit point for clarity. Others disagree e.g. using early "
+                f"returns to short-circuit {func_type_lbl}s if initial "
+                "validation of some sort makes it possible to avoid the bulk of"
+                f" the {func_type_lbl}. Whatever approach you take make sure "
+                f"your {func_type_lbl} is easy to reason about in terms of what"
+                " it returns and where it exits")
     return returns_comment
 
 def _get_exit_comment(func_el, func_type_lbl, *, repeat=False):
@@ -220,20 +189,20 @@ def func_overview(block_dets, *, repeat=False):
     if not repeat:
         args_vs_params = layout(f"""\
 
-            There is often confusion about the difference between arguments and
-            parameters. {overall_func_type_lbl.title()}s define parameters but
-            receive arguments. You can think of parameters as being like car
-            parks and arguments as the cars that fill them. You supply arguments
-            to a {overall_func_type_lbl} depending on its parameters.
-            """)
+        There is often confusion about the difference between arguments and
+        parameters. {overall_func_type_lbl.title()}s define parameters but
+        receive arguments. You can think of parameters as being like car parks
+        and arguments as the cars that fill them. You supply arguments to a
+        {overall_func_type_lbl} depending on its parameters.
+        """)
         if func_type_lbl == conf.METHOD_LBL:
             methods = layout("""\
 
-                Methods are functions that sit directly inside a class
-                definition. Unless they are defined as static methods e.g. using
-                the `@staticmethod` decorator, they take the instance of the
-                class as the first parameter - almost always named `self`. But
-                they are basically functions.
+            Methods are functions that sit directly inside a class definition.
+            Unless they are defined as static methods e.g. using the
+            `@staticmethod` decorator, they take the instance of the class as
+            the first parameter - almost always named `self`. But they are
+            basically functions.
                 """)
         else:
             methods = ''
@@ -273,17 +242,15 @@ def func_len_check(block_dets, *, repeat=False):
         return None
 
     title = layout(f"""\
-
-        ### {overall_func_type_lbl.title()} possibly too long
-
-        """)
+    ### {overall_func_type_lbl.title()} possibly too long
+    """)
     summary_bits = []
     for name, func_lines_n in long_func_dets:
         summary_bits.append(layout(f"""\
 
-            `{name}` has {utils.int2nice(func_lines_n)} lines of code
-            (including comments but with empty lines ignored).
-            """))
+        `{name}` has {utils.int2nice(func_lines_n)} lines of code (including
+        comments but with empty lines ignored).
+        """))
     summary = ''.join(summary_bits)
     if not repeat:
         sometimes_ok = (
@@ -324,26 +291,24 @@ def func_excess_parameters(block_dets, *, repeat=False):
         return None
 
     title = layout(f"""\
-
-        ### Possibly too many {overall_func_type_lbl} parameters
-
-        """)
+    ### Possibly too many {overall_func_type_lbl} parameters
+    """)
     summary_bits = []
     for i, (name, n_args, func_type_lbl) in enumerate(excess_param_dets):
         first = (i == 0)
         summary_bits.append(layout(f"""\
 
-            `{name}` has {n_args:,} parameters.
-
-            """))
+        `{name}` has {n_args:,} parameters.
+        """))
         if first and not repeat:
             summary_bits.append(layout(f"""\
-                Sometimes it is OK for a {func_type_lbl} to have that many but
-                you should consider refactoring the code or collecting related
-                parameters into single parameters e.g. instead of receiving
-                image size arguments separately perhaps you could receive a
-                dictionary of image size argument details.
-                """))
+
+            Sometimes it is OK for a {func_type_lbl} to have that many but you
+            should consider refactoring the code or collecting related
+            parameters into single parameters e.g. instead of receiving image
+            size arguments separately perhaps you could receive a dictionary of
+            image size argument details.
+            """))
     summary = ''.join(summary_bits)
 
     message = {
@@ -351,24 +316,158 @@ def func_excess_parameters(block_dets, *, repeat=False):
     }
     return message
 
-def get_danger_args(func_el):
-    arg_els = func_el.xpath('args/arguments/args/arg')  ## not kwonlyargs so potentially supplied positionally only
-    arg_names = [arg_el.get('arg') for arg_el in arg_els]
+def get_arg_default_issues(func_el, *, get_issue_status_func, include_kw=True):
+    """
+    Look at this function's arguments. Any issues?
+    """
+    posonly_arg_els = func_el.xpath('args/arguments/posonlyargs/arg')
+    arg_els = func_el.xpath('args/arguments/args/arg')
+    all_arg_els = posonly_arg_els + arg_els  ## order matters
+    if include_kw:
+        kwonly_arg_els = func_el.xpath('args/arguments/kwonlyargs/arg')
+        all_arg_els +=  kwonly_arg_els
+    arg_names = [arg_el.get('arg') for arg_el in all_arg_els]
     arg_default_els = func_el.xpath('args/arguments/defaults')
-    danger_statuses = []
-    for arg_default_el in arg_default_els:
-        for child_el in arg_default_el.getchildren():
-            danger_status = get_danger_status(child_el)
-            danger_statuses.append(danger_status)
+    default_els = arg_default_els  ## order matters
+    if include_kw:
+        kw_default_els = func_el.xpath('args/arguments/kw_defaults')
+        default_els += kw_default_els
+    issue_statuses = []
+    for default_el in default_els:
+        for child_el in default_el.getchildren():
+            issue_status = get_issue_status_func(child_el)
+            issue_statuses.append(issue_status)
     ## reversed because defaults are filled in rightwards e.g. a, b=1, c=2
     ## args = a,b,c and defaults=1,2 -> reversed c,b,a and 2,1 -> c: 2, b: 1
     arg_names_reversed = reversed(arg_names)
-    danger_statuses_reversed = reversed(danger_statuses)
-    args_and_danger_statuses = zip(arg_names_reversed, danger_statuses_reversed)
-    danger_args = [(arg, danger_status)
-        for arg, danger_status in args_and_danger_statuses
-        if danger_status is not None]
-    return danger_args
+    issue_statuses_reversed = reversed(issue_statuses)
+    args_and_issue_statuses = reversed(list(
+        zip(arg_names_reversed, issue_statuses_reversed)))  ## back to left-to-right order
+    args_with_issues = [(arg, issue_status)
+        for arg, issue_status in args_and_issue_statuses
+        if issue_status]
+    return args_with_issues
+
+def _get_mutable_default_args(func_el):
+    """
+    Interested in all args, not just non-keyword ones as when looking for danger
+    arguments. We are also interested in all defaults as well.
+    """
+    return get_arg_default_issues(
+        func_el, get_issue_status_func=get_mutable_status, include_kw=True)
+
+@filt_block_advisor(xpath=FUNC_DEFN_XPATH, warning=True)
+def mutable_default(block_dets, *, repeat=False):
+    """
+    Look for use of mutable defaults and warn against use except in rare cases.
+    """
+    func_els = block_dets.element.xpath(FUNC_DEFN_XPATH)
+    overall_func_type_lbl = get_overall_func_type_lbl(func_els)
+    mutable_defaults_dets = []
+    for func_el in func_els:
+        func_type_lbl = get_func_type_lbl(func_el)
+        name = func_el.get('name')
+        mutable_default_args = _get_mutable_default_args(func_el)
+        if mutable_default_args:
+            mutable_defaults_dets.append(
+                (name, func_type_lbl, mutable_default_args))
+    if not mutable_defaults_dets:
+        return None
+
+    title = layout(f"""\
+    ### {overall_func_type_lbl.title()} has mutable default arguments
+    """)
+    brief_summary_bits = []
+    main_summary_bits = []
+    for i, mutable_default_dets in enumerate(mutable_defaults_dets):
+        name, func_type_lbl, mutable_default_args = mutable_default_dets
+        first = (i == 0)
+        nice_pairs = get_nice_pairs(
+            mutable_default_args, pair_glue=' defaults to a ')
+        issue = layout(f"""\
+
+        `{name}` has the following parameters with mutable defaults:
+        {nice_pairs}.
+        """)
+        brief_summary_bits.append(issue)
+        main_summary_bits.append(issue)
+        if first and not repeat:  ## explaining once is enough ;-)
+            main_summary_bits.append(layout(f"""\
+
+            Mutable default arguments are a well-known "gotcha" in Python. They
+            do not behave as most people would expect them to. It is better to
+            have `None` as the default, and inside the function set the name to
+            the desired mutable if it is `None`.
+            """)
+            +
+            layout("""\
+            ## BAD (mutable default)
+            def communicate(person, msg, people=[]):
+                people.append(person)
+                for person in people:
+                    print(f"Emailed {person} with message: '{msg}'")
+
+            ## GOOD (no mutable defaults)
+            def communicate(person, msg, people=None):
+                if people is None:
+                    people = []
+                people.append(person)
+                for person in people:
+                    print(f"Emailed {person} with message: '{msg}'")
+            """, is_code=True)
+            )
+    brief_summary = ''.join(brief_summary_bits)
+    main_summary = ''.join(main_summary_bits)
+    if not repeat:
+        defaults_explained = (
+            layout(f"""\
+
+            {overall_func_type_lbl.title()}s are stored in memory when they are
+            defined, and that includes any default arguments that belong to
+            them. So every time the {overall_func_type_lbl} is called it is from
+            the same piece of memory and the default argument is the same.
+            Calling a {overall_func_type_lbl} multiple times will mean starting
+            from the mutable default as it was left by the previous call - which
+            makes sense because it is one and the same thing.
+
+            Let's use our BAD version of the `communicate` function to see what
+            happens:
+            """)
+            +
+            layout("""\
+            ## BAD (people list will persist across multiple calls)
+            def communicate(person, msg, people=[]):
+                people.append(person)
+                for person in people:
+                    print(f"Emailed {person} with message: '{msg}'")
+
+            communicate('Jack', 'Apples are on special')
+            ## people is empty, so only Jack is emailed
+            # >>> Emailed Jack with message: 'Apples are on special'
+
+            communicate('Jill', 'Tennis this Saturday')
+            ## people started with Jack in it so both Jack AND Jill get emailed!
+            # >>> Emailed Jack with message: 'Tennis this Saturday'  # <---oops!
+            # >>> Emailed Jill with message: 'Tennis this Saturday'
+            """, is_code=True)
+        )
+    else:
+        defaults_explained = ''
+
+    message = {
+        conf.BRIEF: title + brief_summary,
+        conf.MAIN: title + main_summary + defaults_explained,
+    }
+    return message
+
+def _get_positional_danger_args(func_el):
+    """
+    Interested in non-keyword args only. So posonly_args and args respectively.
+    As for defaults, defaults only (ignoring kw_defaults - the only other option
+    - there is no separate posonly_defaults)
+    """
+    return get_arg_default_issues(
+        func_el, get_issue_status_func=get_danger_status, include_kw=False)
 
 @filt_block_advisor(xpath=FUNC_DEFN_XPATH, warning=True)
 def positional_boolean(block_dets, *, repeat=False):
@@ -385,62 +484,62 @@ def positional_boolean(block_dets, *, repeat=False):
     for func_el in func_els:
         func_type_lbl = get_func_type_lbl(func_el)
         name = func_el.get('name')
-        danger_args = get_danger_args(func_el)
+        danger_args = _get_positional_danger_args(func_el)
         if danger_args:
             positional_dets.append((name, func_type_lbl, danger_args))
     if not positional_dets:
         return None
 
     title = layout(f"""\
-
-        ### {overall_func_type_lbl.title()} expects risky positional arguments
-        """)
+    ### {overall_func_type_lbl.title()} expects risky positional arguments
+    """)
     summary_bits = []
     for i, (name, func_type_lbl, danger_args) in enumerate(positional_dets):
         first = (i == 0)
         summary_bits.append(layout(f"""\
-            A partial analysis of `{name}` found the following risky non-
-            keyword (positional) parameters: {danger_args}.
-            """))
+
+        A partial analysis of `{name}` found the following risky non-keyword
+        (positional) parameters: {get_nice_pairs(danger_args, left_quoter='')}.
+        """))
         if first and not repeat:  ## explaining once is enough ;-)
             summary_bits.append(layout(f"""\
 
-                {func_type_lbl.title}s which expect numbers or booleans
-                (True/False) without requiring keywords are risky. They are
-                risky when if the {func_type_lbl} is changed later to have
-                different parameters. For example, greeting(formal=True) is more
-                intelligible than greeting(True). And intelligible code is safer
-                to alter / maintain over time than mysterious code.
-                """))
+            {func_type_lbl.title}s which expect numbers or booleans
+            (`True`/`False`) without requiring keywords are risky. They are
+            risky when if the {func_type_lbl} is changed later to have different
+            parameters. For example, `greeting(formal=True)` is more
+            intelligible than `greeting(True)`. And intelligible code is safer
+            to alter / maintain over time than mysterious code.
+            """))
     summary = ''.join(summary_bits)
     if not repeat:
         asterisk_demo = (
             layout("""\
 
-                Using an asterisk as a pseudo-parameter forces all parameters to
-                the right to be keywords e.g.
-                """)
+            Using an asterisk as a pseudo-parameter forces all parameters to the
+            right to be keywords e.g.
+            """)
             +
-            layout(f"""\
-                def greeting(name, *, formal=False):
-                    ...
-                """, is_code=True)
+            layout("""\
+            def greeting(name, *, formal=False):
+                ...
+            """, is_code=True)
             +
-            layout(f"""\
+            layout("""\
 
-                In this example you couldn't now call the function
-                greeting('Jo', True) - it would need to be greeting('Jo',
-                formal=True)
-                """)
+            In this example you couldn't now call the function `greeting('Jo',
+            True)` - it would need to be `greeting('Jo', formal=True)`
+            """)
         )
         asterisk_explained = layout(f"""\
-            Putting an asterisk in the parameters has the effect of forcing all
-            parameters to the right to be keyword parameters because the
-            asterisk mops up any remaining positional arguments supplied (if
-            any) when the {func_type_lbl} is called. There can't be any other
-            positional arguments, because they have all been handled already, so
-            only keyword parameters are allowed thereafter.
-            """)
+
+        Putting an asterisk in the parameters has the effect of forcing all
+        parameters to the right to be keyword parameters because the asterisk
+        mops up any remaining positional arguments supplied (if any) when the
+        {func_type_lbl} is called. There can't be any other positional
+        arguments, because they have all been handled already, so only keyword
+        parameters are allowed thereafter.
+        """)
     else:
         asterisk_demo = ''
         asterisk_explained = ''
@@ -516,23 +615,21 @@ def docstring_issues(block_dets, *, repeat=False):
         return None
 
     title = layout("""\
-
-        ### Function / Method missing doc string
-
-        """)
+    ### Function / Method missing doc string
+    """)
     example_docstring = layout(f'''\
-        def greet(name, greet_word='Hi'):
-            """
-            Get a greeting for the supplied person.
+    def greet(name, greet_word='Hi'):
+        """
+        Get a greeting for the supplied person.
 
-            :param str name: person being greeted
-            :param str greet_word: the word to start the greeting
-            :return: a greeting message to the person
-            :rtype: str
-            """
-            greeting = f"{{greet_word}} {{name}} - how are you?"
-            return greeting
-        ''', is_code=True)
+        :param str name: person being greeted
+        :param str greet_word: the word to start the greeting
+        :return: a greeting message to the person
+        :rtype: str
+        """
+        greeting = f"{{greet_word}} {{name}} - how are you?"
+        return greeting
+    ''', is_code=True)
     summary_bits = []
     for i, (func_name, func_type_lbl, problem) in enumerate(docstring_issues):
         first = (i == 0)
@@ -541,53 +638,49 @@ def docstring_issues(block_dets, *, repeat=False):
                 summary_bits.append((
                     layout(f"""\
 
-                        #### {func_type_lbl.title()} missing doc string
+                    #### {func_type_lbl.title()} missing doc string
 
-                        `{func_name}` lacks a doc string - you should probably add one.
+                    `{func_name}` lacks a doc string - you should probably add
+                    one.
 
-                        Note - # comments at the top of the {func_type_lbl} do
-                        not work as doc strings. Python completely ignores them.
-                        If you add a proper doc string, however, it can be
-                        accessed by running `help({func_name})` or
-                        `{func_name}.`\_\_doc\_\_. Which is useful when using
-                        this {func_type_lbl} in bigger projects e.g. in an IDE
-                        (Integrated Development Environment).
+                    Note - # comments at the top of the {func_type_lbl} do not
+                    work as doc strings. Python completely ignores them. If you
+                    add a proper doc string, however, it can be accessed by
+                    running `help({func_name})` or `{func_name}.`\_\_doc\_\_.
+                    Which is useful when using this {func_type_lbl} in bigger
+                    projects e.g. in an IDE (Integrated Development
+                    Environment).
 
-                        Here is an example doc string for a simple function
-                        using one of several valid formats:
-
-                        """)
+                    Here is an example doc string for a simple function using
+                    one of several valid formats:
+                    """)
                     +
                     example_docstring
                 ))
             else:
                 summary_bits.append(layout(f"""\
+                #### `{func_name}` lacks a doc string
 
-                    #### `{func_name}` lacks a doc string
-
-                    You should probably add a doc string to `{func_name}`
-                    """))
+                You should probably add a doc string to `{func_name}`
+                """))
         elif problem == DOCSTRING_TOO_SHORT:
             if first and not repeat:
                 summary_bits.append((layout(f"""\
+                #### Function doc string too brief?
 
-                    #### Function doc string too brief?
-
-                    The doc string for `{func_name}` seems a little
-                    short{param_str}. You might want to rework it. Here is an
-                    example using one of several valid formats:
-
-                    """)
+                The doc string for `{func_name}` seems a little
+                short{param_str}. You might want to rework it. Here is an
+                example using one of several valid formats:
+                """)
                 +
                 example_docstring
                 ))
             else:
                 summary_bits.append(layout(f"""\
+                #### Function doc string too brief?
 
-                    #### Function doc string too brief?
-
-                    The doc string for `{func_name}` seems a little short.
-                    """))
+                The doc string for `{func_name}` seems a little short.
+                """))
     summary = ''.join(summary_bits)
 
     message = {
