@@ -1,46 +1,8 @@
 from collections import defaultdict, namedtuple, Counter
 
 from ..advisors import filt_block_advisor
-from .. import conf
-from ..utils import get_python_version, int2nice, layout_comment as layout
-
-def get_string_3_7(comparison_el):
-    string = comparison_el.get('s')
-    if not string:
-        return None
-    return string
-
-def get_string_3_8(comparison_el):
-    val = comparison_el.get('value')
-    if comparison_el.get('type') == 'str':
-        string = val
-    else:
-        string = None
-    return string
-
-def get_num_3_7(comparison_el):
-    num = comparison_el.get('n')
-    if not num:
-        return None
-    return num
-
-def get_num_3_8(comparison_el):
-    val = comparison_el.get('value')
-    if comparison_el.get('type') in ('int', 'float'):
-        num = val
-    else:
-        num = None
-    return num
-
-python_version = get_python_version()
-if python_version in (conf.PY3_6, conf.PY3_7):
-    get_string = get_string_3_7
-    get_num = get_num_3_7
-elif python_version == conf.PY3_8:
-    get_string = get_string_3_8
-    get_num = get_num_3_8
-else:
-    raise Exception(f"Unexpected Python version {python_version}")
+from .. import ast_funcs, conf
+from ..utils import int2nice, layout_comment as layout
 
 IfDets = namedtuple('IfDetails',
     'multiple_conditions, missing_else, if_clauses')
@@ -138,17 +100,16 @@ def _get_if_comment(block_dets):
     """
     ifs_details = get_ifs_details(block_dets)
     if_comment = layout("""
-
-        ### Conditional statement detected
-        """)
+    ### Conditional statement detected
+    """)
     for n, if_details in enumerate(ifs_details, 1):
         counter = '' if len(ifs_details) == 1 else f" {int2nice(n)}"
         if if_details.multiple_conditions:
             n_elifs = if_details.if_clauses.count(ELIF)
             if_comment += layout(f"""\
 
-                `if` statement{counter} has {int2nice(n_elifs)} `elif` clauses
-                """)
+            `if` statement{counter} has {int2nice(n_elifs)} `elif` clauses
+            """)
             if if_details.missing_else:
                 if_comment += " and no `else` clause."
             else:
@@ -159,9 +120,9 @@ def _get_if_comment(block_dets):
         else:
             if_comment += layout(f"""\
 
-                `if` statement{counter} has no extra clauses e.g. `elif`s or an
-                `else`.
-                """)
+            `if` statement{counter} has no extra clauses e.g. `elif`s or an
+            `else`.
+            """)
     return if_comment
 
 @filt_block_advisor(xpath=IF_XPATH)
@@ -178,32 +139,29 @@ def if_else_overview(block_dets, *, repeat=False):
         demo = (
             layout("""\
 
-                When using `if`, `elif`, or `else`, the item evaluated can
-                either be an expression or a name (variable). Sometimes your
-                code can be more readable if the expression is precalculated and
-                a name is supplied instead. For example:
-
-                """)
+            When using `if`, `elif`, or `else`, the item evaluated can either be
+            an expression or a name (variable). Sometimes your code can be more
+            readable if you make an intermediate variable and use that instead.
+            Shorter, more readable, easier to check the parts. For example:
+            """)
             +
             layout("""\
-                if length_matters and len(phrase) > conf.MAX_SANE_LENGTH:
-                    phrase = chop(phrase)
-                """, is_code=True)
+            if length_matters and len(phrase) > conf.MAX_SANE_LENGTH:
+                phrase = chop(phrase)
+            """, is_code=True)
             +
             layout("""\
-
-                is shorter but more complex than:
-
-                """)
+            is shorter but more complex than:
+            """)
             +
             layout("""\
-                too_long = (
-                    length_matters
-                    and len(phrase) > conf.MAX_SANE_LENGTH
-                )
-                if too_long:
-                    phrase = chop(phrase)
-                """, is_code=True)
+            too_long = (
+                length_matters
+                and len(phrase) > conf.MAX_SANE_LENGTH
+            )
+            if too_long:
+                phrase = chop(phrase)
+            """, is_code=True)
         )
     else:
         demo = ''
@@ -236,67 +194,64 @@ def missing_else(block_dets, *, repeat=False):
         counter = (
             '' if len(ifs_dets_missing_else) == 1 else f" {int2nice(i + 1)}")
         summary_bits.append(layout(f"""\
-            `if` block{counter} has `elif` clauses but lacks an `else` clause.
-
-            """))
+        `if` block{counter} has `elif` clauses but lacks an `else` clause.
+        """))
         if first and not repeat:
             summary_bits.append(layout(f"""\
-                If your `elif` clauses are trying to handle all expected cases
-                it is probably best to include an `else` clause as well just in
-                case something unexpected happens.
 
-                Note - `else` clauses with an `if` and only the `if` underneath
-                count as `elif` clauses.
+            If your `elif` clauses are trying to handle all expected cases it is
+            probably best to include an `else` clause as well just in case
+            something unexpected happens.
+
+            Note - `else` clauses with an `if` and only the `if` underneath
+            count as `elif` clauses.
                 """))
     summary = ''.join(summary_bits)
     dets = (
         layout("""\
 
-            You may have left out the `else` because it is currently impossible
-            that this branch will ever be called. You know that you can only
-            receive the items currently handled by the `elif`s so `else` can
-            logically never be called. And that it true - until it isn't later
-            ;-) - for example, if the calling code starts supplying more types
-            of whatever is being evaluated in the `elif`s. This problem happens
-            surprisingly often and can create nasty bugs that are hard to trace.
-            If you add an `else` clause that raises an exception you will
-            instantly know if the expected conditions for your conditional are
-            breached and exactly what to fix.
+        You may have left out the `else` because it is currently impossible that
+        this branch will ever be called. You know that you can only receive the
+        items currently handled by the `elif`s so `else` can logically never be
+        called. And that it true - until it isn't later ;-) - for example, if
+        the calling code starts supplying more types of whatever is being
+        evaluated in the `elif`s. This problem happens surprisingly often and
+        can create nasty bugs that are hard to trace. If you add an `else`
+        clause that raises an exception you will instantly know if the expected
+        conditions for your conditional are breached and exactly what to fix.
 
-            For example:
-
-            """)
+        For example:
+        """)
         +
         layout("""\
-            ## At the point this code is written we absolutely know
-            ## there are only two user types: managers and staff
+        ## At the point this code is written we absolutely know
+        ## there are only two user types: managers and staff
 
-            if user == 'manager':
-                create_alert()
-                set_up_manager(user)
-            elif user == 'staff':
-                set_up_staff(user)
-            else:
-                ## This can logically never happen so
-                ## we might be tempted to leave out the else clause
-                pass
-            """, is_code=True)
+        if user == 'manager':
+            create_alert()
+            set_up_manager(user)
+        elif user == 'staff':
+            set_up_staff(user)
+        else:
+            ## This can logically never happen so
+            ## we might be tempted to leave out the else clause
+            pass
+        """, is_code=True)
         +
         layout("""\
 
-            But then a new user type is created (e.g. admin) and the logic of
-            the entire program breaks all over the place with no clarity about
-            the real source.
+        But then a new user type is created (e.g. admin) and the logic of the
+        entire program breaks all over the place with no clarity about the real
+        source.
 
-            It would be better to have an `else` clause as follows so you know
-            exactly what violated the program's assumptions:
-
-            """)
+        It would be better to have an `else` clause as follows so you know
+        exactly what violated the program's assumptions:
+        """)
         +
         layout("""\
-            else:
-                raise Exception(f"Unexpected user: '{user}'")
-            """, is_code=True)
+        else:
+            raise Exception(f"Unexpected user: '{user}'")
+        """, is_code=True)
         )
 
     message = {
@@ -338,13 +293,13 @@ def get_split_membership_dets(if_el):
         if not comparison_els:
             continue
         comparison_el = comparison_els[0]  ## Str or Num etc
-        comp_val = get_string(comparison_el)
+        comp_val = ast_funcs.str_from_el(comparison_el)
         if comp_val is not None:
             basic_types.add(STRING)
             if len(basic_types) > 1:
                 return None
         else:
-            comp_val = get_num(comparison_el)
+            comp_val = ast_funcs.num_str_from_el(comparison_el)
             if comp_val is not None:
                 basic_types.add(NUMBER)
                 if len(basic_types) > 1:
@@ -394,37 +349,31 @@ def split_group_membership(block_dets, *, repeat=False):
         return None
 
     summary = layout(f"""\
+    ### Possible option of evaluating group membership
 
-            ### Possible option of evaluating group membership
-
-            It looks like `{comp_var}` has been in multiple separate
-            comparisons. In Python it is possible to do a simple check of group
-            membership instead.
-        """)
+    It looks like `{comp_var}` has been in multiple separate comparisons. In
+    Python it is possible to do a simple check of group membership instead.
+    """)
     if not repeat:
         demo = (
-            layout(f"""\
-
-                For example:
-
+            layout("""\
+            For example:
             """)
             +
             layout(f"""\
-                if {comp_var} in {comp_vals_gp_str}:
-                    ...
-                """, is_code=True)
+            if {comp_var} in {comp_vals_gp_str}:
+                ...
+            """, is_code=True)
         )
         extra_demo = (
-            layout(f"""\
-
-                Or to check if a variable is NOT in a group:
-
-                """)
+            layout("""\
+            Or to check if a variable is NOT in a group:
+            """)
             +
             layout(f"""\
-                if {comp_var} not in {comp_vals_gp_str}:
-                    ...
-                """, is_code=True)
+            if {comp_var} not in {comp_vals_gp_str}:
+                ...
+            """, is_code=True)
         )
     else:
         demo = ''
@@ -463,7 +412,7 @@ def get_has_explicit_count(if_el):
     if not comparison_els:
         return False
     comparison_el = comparison_els[0]
-    n = get_num(comparison_el)
+    n = ast_funcs.num_str_from_el(comparison_el)
     if n is None:
         return False
     explicit_booleans = [
@@ -494,43 +443,40 @@ def implicit_boolean_enough(block_dets, *, repeat=False):
     if not implicit_boolean_possible:
         return None
     title = layout("""\
-
-        ### Possible option of using an implicit boolean
-
-        """)
+    ### Possible option of using an implicit boolean
+    """)
     if not repeat:
         summary = (
             layout("""\
-                There is often no need to check a non-zero length explicitly. In
-                Python, "", 0, [], {}, (), `None` all evaluate to False when we
-                ask if they are `True` or `False`. And they all evaluate to
-                `True` if they contain something.
 
-                So if we have a list `my_list` we can replace:
+            There is often no need to check a non-zero length explicitly. In
+            Python, "", 0, [], {}, (), `None` all evaluate to False when we ask
+            if they are `True` or `False`. And they all evaluate to `True` if
+            they contain something.
 
-                """)
+            So if we have a list `my_list` we can replace:
+            """)
             +
             layout("""\
-                if len(my_list) > 0:
-                    ...
+            if len(my_list) > 0:
+                ...
 
-                """, is_code=True)
+            """, is_code=True)
             +
             layout("""\
-                with:
-
-                """)
+            with:
+            """)
             +
             layout("""\
-                if my_list:  ## if empty, evaluates to False otherwise True
-                    ...
+            if my_list:  ## if empty, evaluates to False otherwise True
+                ...
 
-                """, is_code=True)
+            """, is_code=True)
         )
     else:
         summary = layout("""\
-            There is often no need to check a non-zero length explicitly.
-            """)
+        There is often no need to check a non-zero length explicitly.
+        """)
     message = {
         conf.BRIEF: title + summary,
     }
@@ -588,51 +534,45 @@ def short_circuit(block_dets, *, repeat=False):
         return None
 
     summary = layout("""\
+    ### Potential to collapse `if`s (possibly relying on short-circuiting)
 
-        ### Potential to collapse `if`s (possibly relying on short-circuiting)
-
-        This code contains nested `if`s that could potentially be collapsed into
-        one single conditional expression.
-
-        """)
+    This code contains nested `if`s that could potentially be collapsed into one
+    single conditional expression.
+    """)
     if not repeat:
         how2short_circuit = layout("""\
-            If the second `if` can only be run once the first `if` has been
-            evaluated as True it is still possible to combine the two
-            expressions - as long as the two expressions are combined with `and`
-            AND the test to see if the nested expression can be run comes first.
-            We can do this because Python supports "short-circuiting" :-).
-            """)
+
+        If the second `if` can only be run once the first `if` has been
+        evaluated as True it is still possible to combine the two expressions -
+        as long as the two expressions are combined with `and` AND the test to
+        see if the nested expression can be run comes first. We can do this
+        because Python supports "short-circuiting" :-).
+        """)
         demo = (
             layout("""\
-
-                For example, we can rewrite the following:
-
-                """)
+            For example, we can rewrite the following:
+            """)
             +
             layout("""\
-
-                if word is not None:
-                    ## the next expression would raise TypeError: object of type
-                    ## 'NoneType' has no len() if word was None and it got this far
-                    if len(word) > 20:
-                        print(f"'{word}' is a long word")
-
-                """, is_code=True)
-            +
-            layout("""\
-
-                as:
-
-                """)
-            +
-            layout("""\
-                ## the second clause is only evaluated if the first evaluates as True
-                ## i.e. we have relied on short-circuiting
-                if word is not None and len(word) > 20:
+            if word is not None:
+                ## the next expression would raise TypeError: object of type
+                ## 'NoneType' has no len() if word was None and it got this far
+                if len(word) > 20:
                     print(f"'{word}' is a long word")
 
-                """, is_code=True)
+            """, is_code=True)
+            +
+            layout("""\
+            as:
+            """)
+            +
+            layout("""\
+            ## the second clause is only evaluated if the first evaluates as True
+            ## i.e. we have relied on short-circuiting
+            if word is not None and len(word) > 20:
+                print(f"'{word}' is a long word")
+
+            """, is_code=True)
         )
     else:
         how2short_circuit = ''
@@ -694,112 +634,88 @@ def any_all(block_dets, *, repeat=False):
             f"could_any_something ({could_any_something}) "
             f"and could_all_something ({could_all_something})")
     title = layout(f"""\
-
-        ### {title_content}
-
-        """)
+    ### {title_content}
+    """)
     if not repeat:
         summary = layout("""\
 
-            Python has built-in `any` and `all` functions that make your code
-            more readable when you're evaluating whether all or any of a group
-            of items are True (or whether _not_ all or _not_ any are True). The
-            argument has to be an iterable e.g. a list.
-
-            """)
+        Python has built-in `any` and `all` functions that make your code more
+        readable when you're evaluating whether all or any of a group of items
+        are True (or whether _not_ all or _not_ any are True). The argument has
+        to be an iterable e.g. a list.
+        """)
         demo = (
             layout("""\
-
-                For example, instead of:
-
-                """)
+            For example, instead of:
+            """)
             +
             layout("""\
-
-                if current or valid or safe or permitted:
-                    proceed()
-
-                """, is_code=True)
+            if current or valid or safe or permitted:
+                proceed()
+            """, is_code=True)
             +
             layout("""\
-
-                you could write the more semantic:
-
-                """)
+            you could write the more semantic:
+            """)
             +
             layout("""\
-
-                if all([current, valid, safe, permitted]):
-                    proceed()
-
-                """, is_code=True)
+            if all([current, valid, safe, permitted]):
+                proceed()
+            """, is_code=True)
             +
             layout("""\
-
-                or the equivalent, and, arguably, more readable:
-
-                """)
+            or the equivalent, and, arguably, more readable:
+            """)
             +
             layout("""\
+            conditions = [current, valid, safe, permitted]
+            if all(conditions):
+                proceed()
 
-                conditions = [current, valid, safe, permitted]
-                if all(conditions):
-                    proceed()
-
-                """, is_code=True)
+            """, is_code=True)
             +
             layout("""\
-
-                Or if you wanted the negation the syntax would be:
-
-                """)
+            Or if you wanted the negation the syntax would be:
+            """)
             +
             layout("""\
-
-                if not all(conditions):
-                    exit()
-
-                """, is_code=True)
+            if not all(conditions):
+                exit()
+            """, is_code=True)
             +
             layout("""\
-
-                The `any` function works the same way
-
-                """)
+            The `any` function works the same way
+            """)
             +
             layout("""\
+            flatmate_positives = [
+                has_car, has_appliances, has_tv, great_cook, funny]
+            if any(flatmate_positives):
+                recruit_to_flat()
 
-                flatmate_positives = [
-                    has_car, has_appliances, has_tv, great_cook, funny]
-                if any(flatmate_positives):
-                    recruit_to_flat()
-
-                """, is_code=True)
+            """, is_code=True)
         )
         extra = (
             layout("""\
 
-                It is easy to forget that an iterable has to be passed in as the
-                argument rather than the individual items. So:
-
-                """)
+            It is easy to forget that an iterable has to be passed in as the
+            argument rather than the individual items. So:
+            """)
             +
             layout("""\
+            if any(books, magazines, comics, games):  ## FAIL TypeError: any() takes exactly one argument (4 given)
+                relax()
 
-                if any(books, magazines, comics, games):  ## FAIL TypeError: any() takes exactly one argument (4 given)
-                    relax()
-
-                if any([books, magazines, comics, games]):  ## SUCCESS
-                    relax()
-                """, is_code=True)
+            if any([books, magazines, comics, games]):  ## SUCCESS
+                relax()
+            """, is_code=True)
         )
     else:
         summary = layout("""\
 
-            Python has built-in `any` and `all` functions that can make your
-            code more readable.
-
-            """)
+        Python has built-in `any` and `all` functions that can make your code
+        more readable.
+        """)
         demo = ''
         extra = ''
 
