@@ -2,11 +2,14 @@ import logging
 from textwrap import dedent
 
 from .cli_extras import md2cli
-from ..utils import layout_comment as layout
+from ..utils import get_line_numbered_snippet, layout_comment as layout
 
 """
 Note - displays properly in the terminal but not necessarily in other output
-e.g. Eclipse console
+e.g. Eclipse console.
+
+Lots in common with md displayer but risks of DRYing probably outweigh benefits
+at this stage.
 """
 
 from .. import conf
@@ -15,9 +18,9 @@ TERMINAL_WIDTH = 80
 
 MDV_CODE_BOUNDARY = "```"
 
-def get_message(message_dets, message_level):
-    message = dedent(message_dets.message[message_level])
-    if message_level == conf.EXTRA:
+def get_message(message_dets, detail_level):
+    message = dedent(message_dets.message[detail_level])
+    if detail_level == conf.EXTRA:
         message = dedent(message_dets.message[conf.MAIN]) + message
     message = dedent(message)
     message = (message
@@ -27,33 +30,40 @@ def get_message(message_dets, message_level):
     message = md2cli.main(md=message.replace('`', ''))  ## They create problems in formatting
     return message
 
-def _need_snippet_displayed(block_messages_dets, *, multi_block=False):
+def _need_snippet_displayed(overall_messages_dets, block_messages_dets, *,
+        multi_block=False):
     """
     Don't need to see the code snippet displayed when it is already visible:
     * because there is only one block in snippet and there is a block message
-      for it (which will display the block i.e. the entire snippet)
+      for it (which will display the block i.e. the entire snippet) UNLESS there
+      is an overall message separating them
     Otherwise we need it displayed.
     """
     mono_block_snippet = not multi_block
-    if mono_block_snippet and block_messages_dets:
+    if mono_block_snippet and block_messages_dets and not overall_messages_dets:
         return False
     return True
 
 def display(snippet, messages_dets, *,
-        message_level=conf.BRIEF, in_notebook=False, multi_block=False):
+        detail_level=conf.BRIEF,
+        warnings_only=False, in_notebook=False, multi_block=False):
     """
     Show by code blocks.
     """
     logging.debug(f"{__name__} doesn't use in_notebook setting {in_notebook}")
     md2cli.term_columns = TERMINAL_WIDTH
+    if warnings_only:
+        options_msg = conf.WARNINGS_ONLY_MSG
+    else:
+        options_msg = conf.ALL_HELP_SHOWING_MSG
     text = [
         md2cli.main(layout(f"""\
             # SuperHELP - Help for Humans!
 
-            Help is provided for your overall snippet and for each block of code
-            as appropriate.
+            {conf.INTRO}
 
-            Currently showing {message_level} content as requested.
+            Currently showing {detail_level} content as requested.
+            {options_msg}.
 
             {conf.MISSING_ADVICE_MESSAGE}
             """
@@ -61,22 +71,20 @@ def display(snippet, messages_dets, *,
     ]
     overall_messages_dets, block_messages_dets = messages_dets
     display_snippet = _need_snippet_displayed(
-        block_messages_dets, multi_block=multi_block)
+        overall_messages_dets, block_messages_dets, multi_block=multi_block)
     if display_snippet:
+        line_numbered_snippet = get_line_numbered_snippet(snippet)
         text.append(md2cli.main(dedent(
             "## Overall Snippet"
             f"\n{MDV_CODE_BOUNDARY}\n"
-            + snippet
+            + line_numbered_snippet
             + f"\n{MDV_CODE_BOUNDARY}")))
     for message_dets in overall_messages_dets:
-        message = get_message(message_dets, message_level)
+        message = get_message(message_dets, detail_level)
         text.append(message)
     block_messages_dets.sort(key=lambda nt: (nt.first_line_no, nt.warning))
     prev_line_no = None
-    for n, message_dets in enumerate(block_messages_dets):
-        if n > 41:
-            has_prob = True
-            print(has_prob)
+    for message_dets in block_messages_dets:
         ## display code for line number (once ;-))
         line_no = message_dets.first_line_no
         new_block = (line_no != prev_line_no)
@@ -97,7 +105,7 @@ def display(snippet, messages_dets, *,
                 """)))
             block_has_warning_header = True
         ## process message
-        message = get_message(message_dets, message_level)
+        message = get_message(message_dets, detail_level)
         text.append(message)
     content = '\n'.join(text)
     print(content)
